package com.yizhi.application.controller;

import com.alibaba.fastjson.JSON;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.PublicPlatformConfig;
import com.yizhi.application.domain.TrAccountUserinfo;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.IPublicPlatformConfigService;
import com.yizhi.system.application.vo.domain.Account;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.system.remote.SiteClient;
import com.yizhi.util.application.encrypt.ShaEncrypt;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.wechat.application.vo.wechat.Constants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 自定义项目定义届酷狗
 *
 * @author lingye
 * @date 2020-3-23 17:49:01
 */
@RestController
@RequestMapping(value = "/custom/project")
public class CustomProjectController {

    public static final Logger LOGGER = LoggerFactory.getLogger(CustomProjectController.class);

    @Autowired
    private SiteClient siteClient;
    @Autowired
    private AccountClient accountClient;
    @Autowired
    private UserInfoController userInfoController;
    @Autowired
    private IPublicPlatformConfigService iPublicPlatformConfigService;
    @Autowired
    private WeChatUserController weChatUserController;
    @Value("${wechat.account.initializtion.pwd}")
    private String password;

    @Autowired
    IdGenerator idGenerator;

    @GetMapping(value = "/token")
    public Map<String, Object> getToken(@RequestParam(name = "weChatUserId", required = true) String weChatUserId,
                                                      @RequestParam(name = "appId", required = true) String appId,
                                                      @RequestParam(name = "agentId", required = false) String agentId,
                                                      @RequestParam(name = "orgId", required = true) Long orgId
    ) {
        RequestContext context = ContextHolder.get();
        LOGGER.info("请求的上下文信息:{}", JSON.toJSON(context));
        Map<String, Object> retMap = new HashMap<String, Object>(16);
        retMap.put("returnCode", 1);
        //获取企业的信息
        Long siteId = context.getSiteId();
        String siteCode = context.getSiteCode();
        Long companyId = context.getCompanyId();
        String companyCode = context.getCompanyCode();

        TrAccountUserinfo trAccountUserinfo = userInfoController.queryUserInWechat(weChatUserId, companyId);
        PublicPlatformConfig platformConfig = null;
        List<PublicPlatformConfig> pubConfigs = iPublicPlatformConfigService.getPubConfigs(appId, agentId);
        LOGGER.info("公众号配置所有列表：{}", JSON.toJSON(pubConfigs));
        if (pubConfigs == null) {
            retMap.put("returnCode", 2);
            return retMap;
        }
        for (PublicPlatformConfig config : pubConfigs) {
            if (companyCode.equals(config.getCompanyCode()) && siteCode.equals(config.getSiteCode())) {
                platformConfig = config;
                break;
            }
        }
        if (platformConfig == null) {
            // 公众号配置的站点和链接的站点不匹配
            retMap.put("returnCode", 2);
            return retMap;
        }
        LOGGER.info("公众号配置：{}", JSON.toJSON(platformConfig));
        AccountVO accountVO = new AccountVO();
        if (null != trAccountUserinfo) {
            accountVO = accountClient.findById(trAccountUserinfo.getAccountId());
            LOGGER.info("当前用户的信息accountVO：{}", accountVO);
            retMap.put("userInfo", accountVO);
            weChatUserController.addUserLoginLog(companyId, siteId, accountVO, Constants.FIVE);
            return retMap;
        } else {
            Account account = new Account();
            account.setId(idGenerator.generate());
            account.setName(weChatUserId);
            account.setCompanyId(companyId);
            account.setOrgId(orgId);
            account.setPassword(ShaEncrypt.encryptNewPassword(password));
            account.setCreateTime(new Date());
            account.setCreateByName("system");
            Account user = accountClient.saveThirdAccount(account);

            BeanUtils.copyProperties(user, accountVO);
            // 生成站点访问权限
            siteClient.relateAccountSite(siteId, account.getCompanyId(), siteCode);
            //绑定用户
            weChatUserController.bindCompanyWeChatUser(weChatUserId, companyId, siteId, account.getId(),4);
            LOGGER.info("当前用户的信息accountVO：{}", accountVO);
            retMap.put("userInfo", accountVO);
            weChatUserController.addUserLoginLog(companyId, siteId, accountVO, Constants.FIVE);
            return retMap;
        }
    }

}
