package com.yizhi.application.domain;

import com.baomidou.mybatisplus.activerecord.Model;
import com.baomidou.mybatisplus.annotations.TableField;
import com.baomidou.mybatisplus.annotations.TableName;
import com.baomidou.mybatisplus.enums.FieldFill;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.Date;

/**
 * <p>
 * 用户信息表
 * </p>
 *
 * @author lilingye
 * @since 2018-08-20
 */
@Data
@Api(tags = "UserInfo", description = "用户信息表")
@TableName("user_info")
public class UserInfo extends Model<UserInfo> {

    private static final long serialVersionUID = 1L;


    @ApiModelProperty(value = "表唯一标识id")
    private Long id;

    @ApiModelProperty(value = "用户的唯一标识")
    private String wechatuserid;

    @ApiModelProperty(value = "用户昵称")
    private String nickname;

    @ApiModelProperty(value = "性别。0表示未定义，1表示男性，2表示女性")
    private Integer sex;

    @ApiModelProperty(value = "省份")
    private String province;

    @ApiModelProperty(value = "城市")
    private String city;

    @ApiModelProperty(value = "国家")
    private String country;

    @ApiModelProperty(value = "用户头像")
    private String headimgurl;

    @ApiModelProperty(value = "用户特权信息，json 数组，如微信沃卡用户为（chinaunicom）")
    private String privilege;

    @ApiModelProperty(value = "只有在用户将公众号绑定到微信开放平台帐号后，才会出现该字段。")
    @TableField(value = "union_id")
    private String unionId;

    @ApiModelProperty(value = "企业成员手机号，仅在用户同意snsapi_privateinfo授权时返回")
    private String mobile;

    @ApiModelProperty(value = "企业成员邮箱")
    private String email;

    @ApiModelProperty(value = "电话号码")
    private String telephone;

    @ApiModelProperty(value = "英文名称")
    @TableField("english_name")
    private String englishName;

    @ApiModelProperty(value = "职位")
    private String position;

    @ApiModelProperty(value = "企业 员工个人二维码，扫描可添加为外部联系人")
    @TableField(value = "qr_code")
    private String qrCode;

    @ApiModelProperty(value = "1 普通微信用户 0企业微信用户")
    @TableField("user_type")
    private Integer userType;

    @ApiModelProperty(value = "创建时间")
    @TableField(value = "create_time", fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty(value = "0有效，1删除")
    @TableField("is_deleted")
    private Integer isDeleted;

    @ApiModelProperty(value = "第三方应用id")
    @TableField(value = "corp_id",exist = false)
    private String corpId;

    @ApiModelProperty(value = "企业 员工个人二维码，扫描可添加为外部联系人")
    @TableField(value = "user_id",exist = false)
    private String userId;

    @ApiModelProperty(value = "安装应用的设备Id")
    @TableField(value = "device_id",exist = false)
    private String deviceId;


    @Override
    protected Serializable pkVal() {
        return this.id;
    }

}
