package com.yizhi.application.feign;

import com.alibaba.fastjson.JSONObject;
import feign.Headers;
import feign.Param;
import feign.RequestLine;
import org.springframework.stereotype.Service;

@Headers("Content-Type: application/json")
@Service("weChatClient")
public interface WeChatClient {
//    /**
//     * 获取code
//     * @param appid
//     * @param redirect_uri
//     * @param scope
//     * @return
//     */
//    @RequestLine("GET /get")
//    String getCode(@Param("appid") String appid,@Param("redirecct_uri") String redirect_uri,
//                   @Param("scope") String scope);

    /**
     * 获取token
     *
     * @param appid  公众号的唯一标识
     * @param secret 公众号的appsecret
     * @param code
     * @return
     */
    @RequestLine("GET /sns/oauth2/access_token?appid={appid}&secret={secret}&code={code}&grant_type=authorization_code")
    JSONObject getWechaToken(@Param("appid") String appid,
                             @Param("secret") String secret,
                             @Param("code") String code);

    /**
     * 刷新token
     *
     * @param appid         公众号的唯一标识
     * @param refresh_token 填写通过access_token获取到的refresh_token参数
     * @return
     */
    @RequestLine("GET /sns/oauth2/refresh_token?appid={appid}&grant_type=refresh_token&refresh_token={refresh_token}")
    JSONObject refreshToken(@Param("appid") String appid,
                            @Param("refresh_token") String refresh_token);

    /**
     * 获取用户详情
     *
     * @param accessToken 网页授权接口调用凭证,注意：此access_token与基础支持的access_token不同
     * @param openid      用户唯一标识
     * @return
     */
    @RequestLine("GET /sns/userinfo?access_token={accessToken}&openid={openid}&lang=zh_CN")
    JSONObject getUserInfo(@Param("accessToken") String accessToken,
                           @Param("openid") String openid);

    /**
     * 微信公众号获取jsapi_ticket
     *
     * @param accessToken Token 值
     * @return
     */
    @RequestLine("GET /cgi-bin/ticket/getticket?access_token={accessToken}&type=jsapi")
    JSONObject getTicket(@Param("accessToken") String accessToken);

    /**
     * 获取accessToken
     *
     * @param appid
     * @param secret
     * @return
     */
    @RequestLine("GET /cgi-bin/token?grant_type=client_credential&appid={appid}&secret={secret}")
    JSONObject getAccessToken(@Param("appid") String appid, @Param("secret") String secret);


    @RequestLine("GET /cgi-bin/get_current_selfmenu_info?access_token={accessToken}")
    JSONObject getMenuInfo(@Param("accessToken") String accessToken);

}
