package com.yizhi.application.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.application.feign.CWeChatClient;
import com.yizhi.application.service.ICwechatService;
import com.yizhi.application.service.IUserInfoService;
import com.yizhi.wechat.application.vo.wechat.Constants;
import com.yizhi.wechat.application.vo.wechat.WUserInfoVO;
import com.yizhi.wechat.application.vo.wechat.WechatOrg;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author lilingye
 * @date 2018-8-17 14:19:09
 */
@Log4j2
@Service
public class CwechatServiceImpl implements ICwechatService {
    @Autowired
    private IUserInfoService userInfoService;

    @Autowired
    private CWeChatClient cWeChatClient;

    @Override
    public List<String> getCwechatOrg(String accessToken, Long cOrgId) {

        List<String> orgNameList = getOrgName(new ArrayList<String>(), accessToken, cOrgId);
        Collections.reverse(orgNameList);
        log.info("查询到当前组织：" + orgNameList);
        return orgNameList;
    }

    @Override
    public WUserInfoVO getCwechatUserDetail(String accessToken, String userId) {

        JSONObject cWechatUserDetail = cWeChatClient.getCWechatUserDetail(accessToken, userId);
        log.info("获取到的用户的信息：" + cWechatUserDetail.toString());
        WUserInfoVO wUserInfoVO = new WUserInfoVO();

        if ("0".equals(cWechatUserDetail.getString("errcode"))) {
            wUserInfoVO.setWechatuserid(cWechatUserDetail.getString("userid") != null ? cWechatUserDetail.getString("userid") : "");
            wUserInfoVO.setNickname(cWechatUserDetail.getString("name") != null ? cWechatUserDetail.getString("name") : "");
            wUserInfoVO.setMobile(cWechatUserDetail.getString("mobile") != null ? cWechatUserDetail.getString("mobile") : "");
            wUserInfoVO.setPosition(cWechatUserDetail.getString("position") != null ? cWechatUserDetail.getString("position") : "");
            wUserInfoVO.setSex(Integer.parseInt(cWechatUserDetail.getString("gender") != null ? cWechatUserDetail.getString("gender") : ""));
            wUserInfoVO.setHeadimgurl(cWechatUserDetail.getString("avatar") != null ? cWechatUserDetail.getString("avatar") : "");
            wUserInfoVO.setEmail(cWechatUserDetail.getString("email") != null ? cWechatUserDetail.getString("email") : "");
            wUserInfoVO.setEnglishName(cWechatUserDetail.getString("englishname") != null ? cWechatUserDetail.getString("englishname") : "");
            wUserInfoVO.setTelephone(cWechatUserDetail.getString("telephone") != null ? cWechatUserDetail.getString("telephone") : "");
            wUserInfoVO.setQrCode(cWechatUserDetail.getString("qr_code") != null ? cWechatUserDetail.getString("qr_code") : "");
            wUserInfoVO.setMainDepartmentId(cWechatUserDetail.getString("main_department"));
            log.info("拿到的企业主部门信息：{}", cWechatUserDetail.getString("main_department"));
            JSONArray departmentList = cWechatUserDetail.getJSONArray("department");
            log.info("拿到的企业部门信息：{}", JSON.toJSONString(departmentList));
            // 取第一个
//            String departmentId = departmentList.getString(0);
            //2020-09-24改为获取主部门即可
            wUserInfoVO.setDepartmentId(wUserInfoVO.getMainDepartmentId());
        }
        return wUserInfoVO;
    }

    /**
     * 递归获取用户部门list
     *
     * @param orgNameList
     * @param accessToken
     * @param cOrgId
     * @return
     */
    public List<String> getOrgName(List<String> orgNameList, String accessToken, Long cOrgId) {


        JSONObject departmentInfo = cWeChatClient.getCwechatDepartmentList(accessToken, cOrgId);
        log.info("department 信息：" + departmentInfo.toString());

        if ((Constants.ZERO).equals(departmentInfo.getString("errcode"))) {
            JSONArray departmentList = departmentInfo.getJSONArray("department");
//          JsonArray to List
            List<WechatOrg> orgList = JSONObject.parseArray(departmentList.toJSONString(), WechatOrg.class);
            Long id = null;
            String orgName = null;
            Long orgId = null;
            for (WechatOrg wechatOrg : orgList) {
                if (cOrgId.equals(wechatOrg.getId())) {
                    orgId = wechatOrg.getId();
                    id = wechatOrg.getParentId();
                    orgName = wechatOrg.getName();
                    orgNameList.add(orgName);
                    break;
                }
            }
            //不拿根部门
            if (Constants.YES_FLAG !=id ) {
                getOrgName(orgNameList, accessToken, id);
                log.info("组织名称:" + orgId + ";" + orgName);
            }


        }
        return orgNameList;
    }

}
